#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2

# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  :
#
# 2021-07-23: rewritten for CMK 2.0
#

from cmk.gui.i18n import _
from cmk.gui.plugins.wato import (
    HostRulespec,
    rulespec_registry,
)
from cmk.gui.valuespec import (
    Dictionary,
    FixedValue,
    ListOfStrings,
    ListChoice,
)

from cmk.gui.plugins.wato.inventory import (
    RulespecGroupInventory,
)

_removecolumns_inv_cisco_eox = [
    #    ('ProductIDDescription', 'PID Description'),
    ('LinkToProductBulletinURL', 'EOL bulletin URL'),
    ('EndOfSecurityVulSupportDate', 'End of service vulnerability support'),
    ('EndOfSWMaintenanceReleases', 'End of software maintenance releases'),
    ('EndOfRoutineFailureAnalysisDate', 'End of routine failure analysis'),
    ('MigrationProductId', 'Migration PID'),
    ('MigrationInformation', 'Migration information'),
    ('MigrationProductInfoURL', 'Migration PID URL'),
    ('MigrationProductName', 'Migration product name'),
]


def _valuespec_inv_cisco_eox():
    return Dictionary(
        title=_('Cisco EoX staus'),
        elements=[

            ('always_use_serial',
             FixedValue(
                 True,
                 title=_('always use serial number'),
                 help=_('if true, CMK will request Cisco EoX information via serial number (default is use PID)'),
             )),
            ('removecolumns',
             ListChoice(
                 title=_('remove columns'),
                 help=_('remove information from EoX report'),
                 choices=_removecolumns_inv_cisco_eox,
                 default_value=[
                     'EndOfSecurityVulSupportDate',
                     'EndOfSWMaintenanceReleases',
                     'EndOfRoutineFailureAnalysisDate',
                     'MigrationInformation',
                     'MigrationProductInfoURL',
                     'MigrationProductName',
                 ],
             )),
            ('PID_black_list',
             ListOfStrings(
                 title=_('drop Product IDs beginning with'),
                 orientation='horizontal',
                 help=_('there will be no request for the following PID'),
             )),
            ('PID_bad_list',
             ListOfStrings(
                 title=_('request EoX information for the following Product IDs via serial number'),
                 orientation='horizontal',
                 help=_('the EoX request for the following PID will by via serial number'),
             )),
            ('SN_black_list',
             ListOfStrings(
                 title=_('drop entrys with the following serial number(s)'),
                 orientation='horizontal',
                 help=_('there will be noe EoX request for the following serial numbers'),
             )),
        ],
    )


rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupInventory,
        match_type='dict',
        name='inv_parameters:inv_cisco_eox',
        valuespec=_valuespec_inv_cisco_eox,
    ))
